/*
 * File   :     eol.java
 * Author :     Tis Veugen
 * Date   :     1998-11-02
 * Version:     1.0
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class eol extends Frame
   implements ActionListener, ItemListener, TextListener
{
  private Button                mInput;
  private Button                mOutput;
  private Button                mConvert;
  private Button                mRemoveLF;
  private Button                mRemoveCR;

  private CheckboxGroup         mEolGroup;
  private Checkbox              mCRLF;
  private Checkbox              mLF;
  private Checkbox              mCR;

  private TextField             mInputTextField;
  private TextField             mOutputTextField;

  private BufferedReader        mInputFile = null;
  private PrintWriter           mOutputFile = null;

  private DataInputStream       mInremFile = null;
  private DataOutputStream      mOuremFile = null;

  private String                mEol = "\r\n";
  private String                mDirectory = "";
  private boolean               mbRemove = false; 
  private byte                  mByteRemove = '\n';

  public                        eol()
  {
    mInputFile  = null;
    mOutputFile = null;

    setTitle( "EOL converter (v1.1)" );

    GridBagLayout               lGbl = new GridBagLayout();
    setLayout( lGbl );

    mInput   = addButton( "Input" );
    mOutput  = addButton( "Output" );
    mConvert = addButton( "Convert to:" );
    mRemoveLF = addButton( "Remove LF" );
    mRemoveCR = addButton( "Remove CR" );

    mInputTextField  = addTextField();
    mOutputTextField = addTextField();

    mEolGroup = new CheckboxGroup(); 
    mCRLF = addCheckbox( "CRLF", mEolGroup, true );
    mLF   = addCheckbox( "LF",   mEolGroup, false );
    mCR   = addCheckbox( "CR",   mEolGroup, false );

    GridBagConstraints          lGbc = new GridBagConstraints();

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.HORIZONTAL;
    lGbc.weightx = 20;
    lGbc.weighty = 10;

    add( mInput,   lGbc, 0, 0, 1, 1 );
    add( mOutput,  lGbc, 0, 1, 1, 1 );
    add( mConvert, lGbc, 0, 2, 1, 1 );

    lGbc.fill = GridBagConstraints.NONE;
    add( mInputTextField,  lGbc, 1, 0, 3, 1 );
    add( mOutputTextField, lGbc, 1, 1, 3, 1 );
    add( mCRLF, lGbc, 1, 2, 1, 1 );
    add( mLF,   lGbc, 2, 2, 1, 1 );
    add( mCR,   lGbc, 3, 2, 1, 1 );
    add( mRemoveLF, lGbc, 1, 3, 1, 1 );
    add( mRemoveCR, lGbc, 2, 3, 1, 1 );

    addWindowListener( new WindowAdapter()
                       {
                         public void windowClosing(
                         WindowEvent evt )
                         {
                           setVisible( false );
                           System.exit( 0 );
                         }
                       }
                     );

    setLocation( 100, 100 );
    setSize( 400, 150 );
}

  public void                   add(
  Component                     oComponent,
  GridBagConstraints            oGbc,
  int                           x,
  int                           y,
  int                           w,
  int                           h )
  {
    oGbc.gridx = x;
    oGbc.gridy = y;
    oGbc.gridwidth  = w;
    oGbc.gridheight = h;
    add( oComponent, oGbc );
  }

  public Checkbox               addCheckbox(
  String                        oName,
  CheckboxGroup                 oGroup,
  boolean                       bSelected )
  {
    Checkbox                    lCheckbox = new Checkbox( oName, oGroup, bSelected );

    lCheckbox.addItemListener( this );

    return lCheckbox;
  }

  public Button                 addButton(
  String                        oName )
  {
    Button                      lButton = new Button( oName );

    lButton.addActionListener( this );

    return lButton;
  }

  public TextField              addTextField()
  {
    TextField                   lTextField = new TextField( "", 40 );

    lTextField.addTextListener( this );

    return lTextField;
  }

  public void                   ActionInput()
  {
    String                      lFilename;
    String                      lExtension;

    lFilename = SetFile( "*.*", "Input file", FileDialog.LOAD );
    if ( lFilename != null )
    {
      mInputTextField.setText( lFilename );
      mOutputTextField.setText( lFilename );
    }
  }

  private void                  CloseFiles()
  {
    if ( mInputFile != null )
    {
      try
      {
        mInputFile.close();
      }
      catch ( Exception e )
      {
        System.err.println( "Can't close Input file" );
      }
      mInputFile  = null;
    }

    if ( mOutputFile != null )
    {
      try
      {
        mOutputFile.close();
      }
      catch ( Exception e )
      {
        System.err.println( "Can't close Output file" );
      }
      mOutputFile = null;
    }

    if ( mInremFile != null )
    {
      try
      {
        mInremFile.close();
      }
      catch ( Exception e )
      {
        System.err.println( "Can't close Input file" );
      }
      mInremFile  = null;
    }

    if ( mOuremFile != null )
    {
      try
      {
        mOuremFile.close();
      }
      catch ( Exception e )
      {
        System.err.println( "Can't close Output file" );
      }
      mOuremFile = null;
    }
  }

  private String                OpenFiles(
  String                        oInputFilename,
  String                        oOutputFilename )
  {
    if ( ( oInputFilename == null      )
      || ( oInputFilename.equals( "" ) ) )
    {
      return " No Input file";
    }
    try
    {
      if ( mbRemove )
      {
        mInremFile = new DataInputStream(
                     new BufferedInputStream(
                     new FileInputStream( oInputFilename ) ) );
      }
      else
      {
        mInputFile = new BufferedReader(
                     new FileReader( oInputFilename ) );
      }
    }
    catch ( Exception e )
    {
      return "Can't open Input file " + oInputFilename;
    }

    if ( ( oOutputFilename == null      )
      || ( oOutputFilename.equals( "" ) ) )
    {
      return " No Output file";
    }

    try
    {
      if ( mbRemove )
      {
        mOuremFile = new DataOutputStream(
                     new BufferedOutputStream(
                     new FileOutputStream( oOutputFilename ) ) );
      }
      else
      {
        mOutputFile = new PrintWriter(
                      new FileWriter( oOutputFilename, false ) );
      }
    }
    catch ( Exception e )
    {
      return "Can't open Output file " + oOutputFilename;
    }

    return null;
  }

  private String                SetFile(
  String                        oFilter,
  String                        oHeader,
  int                           iMode )
  {
    FileDialog                  lFd;
    String                      lFilename;

    lFd = new FileDialog( this      // mFrame
                        , oHeader
                        , iMode );
    lFd.setDirectory( mDirectory );
    lFd.setFile( oFilter );
    lFd.show();

    lFilename = lFd.getFile();
    if ( lFilename != null )
    {
      mDirectory = lFd.getDirectory();   
      lFilename = mDirectory + lFilename;
    }

    return lFilename;
  }

  private void                  PrintDialog(
  boolean                       bDialog,
  Frame                         oFrame,
  String                        oHeader,
  String                        oText )
  {
    if ( bDialog )
    {
      new PbnDialog( oFrame, oHeader, oText );
    }
    else
    {
      System.out.println( oText );
    }
  }

  private boolean               ExecuteConvert(
  BufferedReader                oInputFile,
  PrintWriter                   oOutputFile )
  {
    String                      lInput;

    try
    {
      while ( (lInput = mInputFile.readLine()) != null )
      {
        mOutputFile.write( lInput + mEol );
      }
    }
    catch ( Exception e )
    {
      return false;
    }

    return true;
  }

  private boolean               ExecuteRemove(
  DataInputStream               oInremFile,
  DataOutputStream              oOuremFile )
  {
    byte                        b;

    try
    {
      for ( ; ; )
      {
        b = oInremFile.readByte();
        if ( b != mByteRemove )
        {
          oOuremFile.writeByte( b );
        }
      }
    }
    catch ( Exception e )
    {
      if ( ! (e instanceof EOFException) )
      {
        return false;
      }
    }

    return true;
  }

  private boolean               MyRename(
  String                        oSourceFilename,
  String                        oTargetFilename )
  {
    File                        lSourceFile = new File( oSourceFilename );
    File                        lTargetFile = new File( oTargetFilename );

    try
    {
      lTargetFile.delete();
      lSourceFile.renameTo( lTargetFile );
    }
    catch ( Exception e )
    {
      return false;
    }
    return true;
  }

  private boolean               ActionConvert(
  String                        oInputFilename,
  String                        oOutputFilename,
  String                        oEol,
  boolean                       bDialog )
  {
    String                      lString;
    boolean                     bInIsOut = false;
    String                      lInIsOutName = "eol.tmp";
    boolean                     bOK = false;


    if ( ( oOutputFilename != null                   )
      && ( oInputFilename.equals( oOutputFilename ) ) )
    {
      bInIsOut = true;
      if ( lInIsOutName.equals( oOutputFilename ) )
      {
        lInIsOutName = "eol.tmq";
      }
      oOutputFilename = lInIsOutName;
    }

    lString = OpenFiles( oInputFilename
                       , oOutputFilename );
    if ( lString != null )
    {
      PrintDialog( bDialog, this, "Files", lString );
    }
    else
    {
      if ( bDialog )
      {
        this.setCursor( WAIT_CURSOR );
      }

      if ( mbRemove )
      {
        bOK = ExecuteRemove( mInremFile, mOuremFile );
      }
      else
      {
        bOK = ExecuteConvert( mInputFile, mOutputFile );
      }

      if ( bDialog )
      {
        this.setCursor( DEFAULT_CURSOR );
      }
    }

    CloseFiles();
    if ( bOK && bInIsOut )
    { /*
       * Rename the temporary file to the Output file.
       */
      if ( ! MyRename( lInIsOutName, oInputFilename ) )
      {
        bOK = false;
        PrintDialog( bDialog, this, "File error"
                   , "Can't Output to Input file" );
      }
    }

    return bOK;
  }

  private void                  ActionConverts(
  String                        oInputFilename,
  String                        oEol,
  boolean                       bDialog )
  {
    File                        lFile = new File( oInputFilename );
    String                      lPattern = lFile.getName();
    String                      lPath;
    File                        lDirectory;
    String []                   lFiles;
    String []                   lMatchedFiles;
    boolean                     bOK;

    try
    {
      lPath = lFile.getParent();
      lDirectory = new File( lPath );
      lFiles = lDirectory.list();
      if ( lFiles == null )
      {
        PrintDialog( bDialog, this, "Conversion error"
                   , "No files" );
        
        return;
      }
    }
    catch ( Exception e )
    {
      System.err.println( "Can't construct directory" );
      return;
    }

    lMatchedFiles = PatternMatch.MatchList( lFiles, lPattern );

    for ( int i = 0; i < lMatchedFiles.length; i++ )
    {
      File lConvertFile = new File( lDirectory, lMatchedFiles[ i ] );

      bOK = ActionConvert( lConvertFile.toString()
                         , lConvertFile.toString()
                         , oEol
                         , bDialog );

      if ( ! bOK )
      {
        PrintDialog( bDialog, this, "Conversion error"
                   , "Can't execute conversion of file " + lMatchedFiles[ i ] );
      }
    }
  }

  private void                  DoAction()
  {
      String                    lInputFilename;
      lInputFilename = mInputTextField.getText();

      boolean                   bOK;

      if ( ( lInputFilename.indexOf( "*" ) == -1 )
        && ( lInputFilename.indexOf( "?" ) == -1 ) )
      {
        bOK = ActionConvert( lInputFilename
                           , mOutputTextField.getText()
                           , mEol
                           , true );
        if ( ! bOK )
        {
          PrintDialog( true, this, "File error", "Can't execute conversion" );
        }
        else
        {
          PrintDialog( true, this, "Conversion Result", "Ready" );
        }
      }
      else
      {
        ActionConverts( lInputFilename
                      , mEol
                      , true );
        PrintDialog( true, this, "Conversion Result", "Ready" );
      }
  }

  public void                   actionPerformed(
  ActionEvent                   evt )
  {
    String                      lName = evt.getActionCommand();
    String                      lFilename;

    if ( lName.equals( "Input" ) )
    {
      ActionInput();
    }
    else
    if ( lName.equals( "Output" ) )
    {
      lFilename = SetFile( "*.*", "Output file", FileDialog.LOAD );
      if ( lFilename != null )
      {
        mOutputTextField.setText( lFilename );
      }
    }
    else
    if ( lName.equals( "Convert to:" ) )
    {
      mbRemove = false;
      DoAction();
    }
    else
    if ( lName.equals( "Remove CR" ) )
    {
      mbRemove = true;
      mByteRemove = '\r';
      DoAction();
    }
    else
    if ( lName.equals( "Remove LF" ) )
    {
      mbRemove = true;
      mByteRemove = '\n';
      DoAction();
    }
  }

  public void                   itemStateChanged(
  ItemEvent                     evt )
  {
    if ( evt.getStateChange() == ItemEvent.DESELECTED )
    {
      return;
    }
    if ( evt.getItem().equals( "CRLF" ) )
    {
      mEol = "\r\n";
    }
    else
    if ( evt.getItem().equals( "LF" ) )
    {
      mEol = "\n";
    }
    else
    if ( evt.getItem().equals( "CR" ) )
    {
      mEol = "\r";
    }
  }

  public void                   textValueChanged(
  TextEvent                     evt )
  {
  }

  public static void            main(
  String []                     args )
  {
    Frame                       lEol;

    lEol = new eol();
    lEol.show();
  }
}

class PbnDialog extends Dialog
{
  public                        PbnDialog(
  Frame                         oParent,
  String                        oHeader,
  String                        oText )
  {
    super( oParent, oHeader, true );

    Panel                       lPanel = new Panel();
    int                         NrLines;
    int                         Index;

    for ( NrLines = 0; ; NrLines++ )
    {
      Index = oText.indexOf( "\n" );
      if ( Index < 0 )
      {
        lPanel.add( new Label( oText ) );
        break;
      }
      lPanel.add( new Label( oText.substring( 0, Index ) ) );
      oText = oText.substring( Index+1 );
    }
    add( lPanel, "Center" );

    Button                      mOk = new Button( "OK" );
    lPanel = new Panel();
    lPanel.add( mOk );
    add( lPanel, "South" );

    mOk.addActionListener( new ActionListener()
                           {
                             public void actionPerformed(
                             ActionEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

        addWindowListener( new WindowAdapter()
                           {
                             public void windowClosing(
                             WindowEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

    setLocation( 150, 150 );
    setSize( 300, 120 + NrLines * 30 );
    show();
  }
}
